﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.IO;
using Word = Microsoft.Office.Interop.Word;
using Excel = Microsoft.Office.Interop.Excel;
using PowerPoint = Microsoft.Office.Interop.PowerPoint;
using Microsoft.Office.Core;
using System.Diagnostics;

namespace UpLoadFile.Helper
{
    public class FileConvert
    {
        public void FilesConvert(string filename)
        {
            string[] suffix = filename.Split('.');
            var sourcePath = AppDomain.CurrentDomain.BaseDirectory + "files\\" + filename;
            var pdfName = filename.Substring(0, filename.LastIndexOf(".")) + ".pdf";
            var targetPath = AppDomain.CurrentDomain.BaseDirectory + "SWF\\" + pdfName;
            if (suffix[suffix.Length-1] == "pdf")
            {
                File.Copy(sourcePath, targetPath);  
            }
            if (suffix[suffix.Length - 1] == "doc" || suffix[suffix.Length - 1] == "docx" || suffix[suffix.Length - 1] == "txt" || suffix[suffix.Length - 1] == "wps" || suffix[suffix.Length - 1] == "rtf")
            {
                WordConvert(sourcePath, targetPath, Word.WdExportFormat.wdExportFormatPDF);
            }
            if (suffix[suffix.Length - 1] == "xls" || suffix[suffix.Length - 1] == "xlsx" || suffix[suffix.Length - 1] == "et")
            {
                ExcelConvert(sourcePath, targetPath, Excel.XlFixedFormatType.xlTypePDF);
            }
            if (suffix[suffix.Length - 1] == "ppt" || suffix[suffix.Length - 1] == "pptx" || suffix[suffix.Length - 1] == "pot" || suffix[suffix.Length - 1] == "pps" || suffix[suffix.Length - 1] == "dps")
            {
                PowerPointConvert(sourcePath, targetPath, PowerPoint.PpSaveAsFileType.ppSaveAsPDF);
            }
            Pdf2Swf(targetPath, pdfName);
        }

        public bool WordConvert(string sourcePath, string targetPath, Word.WdExportFormat exportFormat)
        {
            bool result;
            object paramMissing = Type.Missing;
            Word.ApplicationClass wordApplication = new Word.ApplicationClass();
            Word.Document wordDocument = null;
            try
            {
                object paramSourceDocPath = sourcePath;
                string paramExportFilePath = targetPath;

                Word.WdExportFormat paramExportFormat = exportFormat;
                bool paramOpenAfterExport = false;
                Word.WdExportOptimizeFor paramExportOptimizeFor =
                        Word.WdExportOptimizeFor.wdExportOptimizeForPrint;
                Word.WdExportRange paramExportRange = Word.WdExportRange.wdExportAllDocument;
                int paramStartPage = 0;
                int paramEndPage = 0;
                Word.WdExportItem paramExportItem = Word.WdExportItem.wdExportDocumentContent;
                bool paramIncludeDocProps = true;
                bool paramKeepIRM = true;
                Word.WdExportCreateBookmarks paramCreateBookmarks =
                        Word.WdExportCreateBookmarks.wdExportCreateWordBookmarks;
                bool paramDocStructureTags = true;
                bool paramBitmapMissingFonts = true;
                bool paramUseISO19005_1 = false;  

                wordDocument = wordApplication.Documents.Open(
                        ref paramSourceDocPath, ref paramMissing, ref paramMissing,
                        ref paramMissing, ref paramMissing, ref paramMissing,
                        ref paramMissing, ref paramMissing, ref paramMissing,
                        ref paramMissing, ref paramMissing, ref paramMissing,
                        ref paramMissing, ref paramMissing, ref paramMissing,
                        ref paramMissing);

                if (wordDocument != null)
                    wordDocument.ExportAsFixedFormat(paramExportFilePath,
                            paramExportFormat, paramOpenAfterExport,
                            paramExportOptimizeFor, paramExportRange, paramStartPage,
                            paramEndPage, paramExportItem, paramIncludeDocProps,
                            paramKeepIRM, paramCreateBookmarks, paramDocStructureTags,
                            paramBitmapMissingFonts, paramUseISO19005_1,
                            ref paramMissing);
                result = true;
            }
            finally
            {
                if (wordDocument != null)
                {
                    wordDocument.Close(ref paramMissing, ref paramMissing, ref paramMissing);
                    wordDocument = null;
                }
                if (wordApplication != null)
                {
                    wordApplication.Quit(ref paramMissing, ref paramMissing, ref paramMissing);
                    wordApplication = null;
                }
                GC.Collect();
                GC.WaitForPendingFinalizers();
                GC.Collect();
                GC.WaitForPendingFinalizers();
            }
            return result;
        }

        //Excel转换方法
        public bool ExcelConvert(string sourcePath, string targetPath, Excel.XlFixedFormatType targetType)
        {
            bool result;
            object missing = Type.Missing;
            Excel.ApplicationClass application = null;
            Excel.Workbook workBook = null;
            try
            {
                application = new Excel.ApplicationClass();
                object target = targetPath;
                object type = targetType;
                workBook = application.Workbooks.Open(sourcePath, missing, missing, missing, missing, missing,
                        missing, missing, missing, missing, missing, missing, missing, missing, missing);

                workBook.ExportAsFixedFormat(targetType, target, Excel.XlFixedFormatQuality.xlQualityStandard, true, false, missing, missing, missing, missing);
                result = true;
            }
            catch
            {
                result = false;
            }
            finally
            {
                if (workBook != null)
                {
                    workBook.Close(true, missing, missing);
                    workBook = null;
                }
                if (application != null)
                {
                    application.Quit();
                    application = null;
                }
                GC.Collect();
                GC.WaitForPendingFinalizers();
                GC.Collect();
                GC.WaitForPendingFinalizers();
            }
            return result;
        }

        //PowerPoint转换方法
        public bool PowerPointConvert(string sourcePath, string targetPath, PowerPoint.PpSaveAsFileType targetFileType)
        {
            bool result;
            object missing = Type.Missing;
            PowerPoint.ApplicationClass application = null;
            PowerPoint.Presentation persentation = null;
            try
            {
                application = new PowerPoint.ApplicationClass();
                persentation = application.Presentations.Open(sourcePath, MsoTriState.msoTrue, MsoTriState.msoFalse, MsoTriState.msoFalse);
                persentation.SaveAs(targetPath, targetFileType, Microsoft.Office.Core.MsoTriState.msoTrue);

                result = true;
            }
            catch
            {
                result = false;
            }
            finally
            {
                if (persentation != null)
                {
                    persentation.Close();
                    persentation = null;
                }
                if (application != null)
                {
                    application.Quit();
                    application = null;
                }
                GC.Collect();
                GC.WaitForPendingFinalizers();
                GC.Collect();
                GC.WaitForPendingFinalizers();
            }
            return result;
        }

        //转换swf文件
        public void Pdf2Swf(string path, string pdfName)
        {
            string cmdStr = AppDomain.CurrentDomain.BaseDirectory + "FlexPaper\\pdf2swf.exe";
            string SourcePath = @"""" + path + @"""";
            string TargetPath = @"""" + AppDomain.CurrentDomain.BaseDirectory + "SWF\\" + pdfName.Substring(0, pdfName.LastIndexOf(".")) + ".swf" + @"""";
            string argsStr = "  -t " + SourcePath + " -s flashversion=9 -o " + TargetPath;
            //调用新进程 进行转换
            Process p = new Process();
            ProcessStartInfo psi = new ProcessStartInfo(cmdStr, argsStr);
            p.StartInfo = psi;
            p.Start();
            p.WaitForExit();
            File.Delete(path);
        }
    }
}